;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Bubble Shark by Tradware
;;  Copyright (C) 2020
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;;
;; PHYSICS SYSTEM
;;

.include "cpctelera.h.s" 
.include "physics_system.h.s"
.include "cpct_functions.h.s"
.include "entity_manager.h.s"
.include "render_system.h.s"

enemigoBala: .db #06, #00, #00, #0x0C, #02, #00, #08, #08, #00, #00
enemigoBalaIzq: .db #07, #00, #00, #0x0C, #02, #00, #08, #08, #00, #00
vel: .db 01

sys_physics_update::
        ld c, #0x01             ;; tipo
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jr nz, salta

        ld a, #0x050            ;; max width pantalla
        sub e_w(ix)
        ld c, a

        ld a, e_x(ix)
        ld e_antx(ix), a
        add e_vel(ix)
        cp c
        jr nc, invalid_x

        valid_x:
            ld e_x(ix), a
            jr endif_x
        invalid_x:     ;; cambio la velocidad de signo, invierto para rebotar
            ld a, e_vel(ix)
            neg
            ld e_vel(ix), a
        endif_x:

        ld a, #0x9C                 ;; max height pantalla
        sub e_h(ix)
        ld c, a

        ld a, e_y(ix)
        ld e_anty(ix), a
        add e_vely(ix)
        add e_vely(ix)
        cp c
        jr nc, invalid_y

        
        ld a, #0x2C                ;; max height pantalla
        ld c, a

        ld a, e_y(ix)
        ld e_anty(ix), a
        add e_vely(ix)
        add e_vely(ix)
        cp c
        jr c, invalid_y

        valid_y:
            ld e_y(ix), a
            jr endif_y
        invalid_y:      ;; cambio la velocidad de signo, invierto para rebotar
            ld a, e_vely(ix)
            neg
            ld e_vely(ix), a

        endif_y:

        salta:
            
ret

sys_physics_update_bala::
    _renloop2:
        push af

        ld c, #0x03
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jr nz, salta2

        ld a, #0x047
        ld c, a
        ld a, (vel)
        ld b, a
        ld a, e_y(ix)
        ld e_anty(ix), a
        ld a, e_x(ix)
        ld e_antx(ix), a
        add b
        add b
        cp c
        jr nc, invalidBala_x

        validBala_x:
            ld e_x(ix), a
            jr salta2

        invalidBala_x:

            call rendersys_borraBalaInvalida

            call entityman_getLastElement_ptr
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                    bucleBala:
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucleBala

                ld a, (hl)
                ld e_flag(ix), a
                ld (hl), #0x00
                inc hl
                ld a, e_x(ix)
                ld a, (hl)
                ld e_x(ix), a
                inc hl
                ld a, e_y(ix)
                ld a, (hl)
                ld e_y(ix), a
                inc hl
                ld a, (hl)
                ld e_color(ix), a
                inc hl
                ld a, (hl)
                ld e_vel(ix), a
                inc hl
                ld a, (hl)
                ld e_vely(ix), a
                inc hl
                ld a, (hl)
                ld e_h(ix), a
                inc hl
                ld a, (hl)
                ld e_w(ix), a
                inc hl
                ld a, (hl)
                ld e_antx(ix), a
                inc hl
                ld a, (hl)
                ld e_anty(ix), a
                inc hl

                ;; AQUI TERMINA LA ELIMINACION DEL OBSTACULO
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                bucleBalaa2:
                        dec ix
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucleBalaa2

                call entityman_setLastElement_ptr
                call entityman_destroyBala
                
                
        salta2:
            pop af
            dec a
            ret z
            ld bc, #sizeof_e
            add ix, bc

    jp _renloop2

    ret


sys_physics_update_item::
    _renloopItem:
        push af

        ld c, #0x04
        ld a, e_flag(ix)
        sub c
        jr nz, saltaItem

        ld a, #0x001 ;; con esto digo donde se tienen que destruir, osea en el 00 de la X
        ld c, a
        ld a, (vel)
        ld b, a
        ld a, e_y(ix)
        ld e_anty(ix), a
        ld a, e_x(ix)
        ld e_antx(ix), a
        sub b
        cp c
        jr c, invalidItem_x

        validItem_x:
            ld e_x(ix), a
            jr saltaItem

        invalidItem_x:
            call rendersys_borraBalaInvalida
            call entityman_getLastElement_ptr
                ld a, #sizeof_e  
                    bucleItem:
                        dec hl
                        dec a
                    jr nz, bucleItem

                ld a, (hl)
                ld e_flag(ix), a
                ld (hl), #0x00
                inc hl
                ld a, e_x(ix)
                ld a, (hl)
                ld e_x(ix), a
                inc hl
                ld a, e_y(ix)
                ld a, (hl)
                ld e_y(ix), a
                inc hl
                ld a, (hl)
                ld e_color(ix), a
                inc hl
                ld a, (hl)
                ld e_vel(ix), a
                inc hl
                ld a, (hl)
                ld e_vely(ix), a
                inc hl
                ld a, (hl)
                ld e_h(ix), a
                inc hl
                ld a, (hl)
                ld e_w(ix), a
                inc hl
                ld a, (hl)
                ld e_antx(ix), a
                ld c, a
                inc hl
                ld a, (hl)
                ld e_anty(ix), a
                ld b, a
                inc hl

                ;; AQUI TERMINA LA ELIMINACION DEL OBSTACULO
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                bucleItm:
                        dec ix
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucleItm

                call entityman_setLastElement_ptr
                call entityman_destroyItem
               
                
        saltaItem:
            pop af
            dec a
            ret z
            ld bc, #sizeof_e
            add ix, bc

    jp _renloopItem
ret

sys_physics_update_obstaculo::
    _renloopObstaculo:
        push af

        ld c, #0x02
        ld a, e_flag(ix)
        sub c
        jr z, saltaObs

        ld c, #0x08
        ld a, e_flag(ix)
        sub c
        jr nz, saltaObstaculo

        saltaObs:
        ld a, #0x001 ;; con esto digo donde se tienen que destruir, osea en el 00 de la X
        ld c, a
        ld a, (vel)
        ld b, a
        ld a, e_y(ix)
        ld e_anty(ix), a
        ld a, e_x(ix)
        ld e_antx(ix), a
        sub b 
        cp c
        jr c, invalidObstaculo_x

        validObstaculo_x:
            ld e_x(ix), a
            jr saltaObstaculo

        invalidObstaculo_x:
        call rendersys_borraBalaInvalida

            call entityman_getLastElement_ptr
                ld a, #sizeof_e  
                    bucleObstaculo:
                        dec hl  
                        dec a
                    jr nz, bucleObstaculo

                ld a, (hl)
                ld e_flag(ix), a
                ld (hl), #0x00
                inc hl
                ld a, (hl) 
                ld e_x(ix), a
                inc hl
                ld a, (hl)
                ld e_y(ix), a
                inc hl
                ld a, (hl)
                ld e_color(ix), a
                inc hl
                ld a, (hl)
                ld e_vel(ix), a
                inc hl
                ld a, (hl)
                ld e_vely(ix), a
                inc hl
                ld a, (hl)
                ld e_h(ix), a
                inc hl
                ld a, (hl)
                ld e_w(ix), a
                inc hl
                ld a, (hl)
                ld e_antx(ix), a
                inc hl
                ld a, (hl)
                ld e_anty(ix), a
                inc hl

                ;; AQUI TERMINA LA ELIMINACION DEL OBSTACULO
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                bucleBala2:
                        dec ix
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucleBala2

                call entityman_setLastElement_ptr         
                call entityman_destroyObstaculo
            

        saltaObstaculo:
            pop af
            dec a
            ret z
            ld bc, #sizeof_e
            add ix, bc

    jp _renloopObstaculo
ret

sys_physics_update_enemigo::
    _renloopenemigo2:
        push af

        ld c, #0x05 
        ld a, e_flag(ix)
        sub c
        jr nz, saltaenemigo2

        ld a, #0x040 ;; width
        add e_w(ix)
        ld c, a
        ld a, e_x(ix)
        ld e_antx(ix), a
        sub e_vel(ix)
        cp c
        jr nc, invalid_xenemigo

        valid_xenemigo:
            ld e_x(ix), a
            jr endif_xenemigo
        invalid_xenemigo:
            ld a, e_vel(ix)
            neg
            ld e_vel(ix), a
        endif_xenemigo:

        ld a, #0x9C;; height
        sub e_h(ix)
        ld c, a
        ld a, e_y(ix)
        ld e_anty(ix), a
        add e_vely(ix)
        cp c
        jr nc, invalid_yenemigo

        ld a, #0x2C                ;; max height pantalla
        ld c, a

        ld a, e_y(ix)
        ld e_anty(ix), a
        add e_vely(ix)
        cp c
        jr c, invalid_yenemigo

        valid_yenemigo:
            ld e_y(ix), a
            jr endif_yenemigo
        invalid_yenemigo: 
            ld a, e_vely(ix)
            neg
            ld e_vely(ix), a
        endif_yenemigo:

        saltaenemigo2:
            pop af
            dec a
            ret z
            ld bc, #sizeof_e
            add ix, bc

    jp _renloopenemigo2
ret

sys_physics_update_balaEnemigo::
    _renloopenemigo78:
        push af

        ld c, #0x06
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jp nz, saltaizq2

        

        ld a, #0x045 ;; con esto digo donde se tienen que destruir, osea en el 00 de la X
        ld c, a

        ld a, e_y(ix)
        ld e_anty(ix), a
        ld a, e_x(ix)
        ld e_antx(ix), a
        add e_vel(ix)  
        cp c
        jr nc, invalidBalaEnemigo_x
        jp validBalaEnemigo_x

        saltaizq2:
        ld c, #0x07
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jp nz, salta2Enemigo
        
        ld a, #0x002 ;; con esto digo donde se tienen que destruir, osea en el 00 de la X
        ld c, a

        ld a, e_y(ix)
        ld e_anty(ix), a
        ld a, e_x(ix)
        ld e_antx(ix), a
        sub e_vel(ix)  ;; TODO: CAMBIAR DIRECCION DE MOVIMIENTO DE LOS OBSTACULOS
        cp c
        jr c, invalidBalaEnemigo_x


        validBalaEnemigo_x:
            ld e_x(ix), a
            jr endifBalaEnemigo_x

        invalidBalaEnemigo_x:

            call rendersys_borraBalaInvalida

            call entityman_getLastElement_ptr
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                    bucleBalaEnemigo:
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucleBalaEnemigo

                ld a, (hl)
                ld e_flag(ix), a
                ld (hl), #0x00
                inc hl
                ld a, e_x(ix)
                ;;ld c, a
                ld a, (hl)
                ld e_x(ix), a
                inc hl
                ld a, e_y(ix)
                ;;ld b, a  ;; guardamos e_y en c para borrar de color del fondo mas adelante
                ld a, (hl)
                ld e_y(ix), a
                inc hl
                ld a, (hl)
                ld e_color(ix), a
                inc hl
                ld a, (hl)
                ld e_vel(ix), a
                inc hl
                ld a, (hl)
                ld e_vely(ix), a
                inc hl
                ld a, (hl)
                ld e_h(ix), a
                inc hl
                ld a, (hl)
                ld e_w(ix), a
                inc hl
                ld a, (hl)
                ld e_antx(ix), a
                inc hl
                ld a, (hl)
                ld e_anty(ix), a
                inc hl

                ld a, #sizeof_e  
                bucleBalaEnemigo2:
                        dec ix
                        dec hl 
                        dec a
                    jr nz, bucleBalaEnemigo2

                call entityman_setLastElement_ptr
                call entityman_destroyBalaEnemigo
                
        endifBalaEnemigo_x:
        
        salta2Enemigo:
            pop af
            dec a
            ret z
            ld bc, #sizeof_e
            add ix, bc

    jp _renloopenemigo78

    ret

dispara_enemigo::
    _loopEnemigoDispara:
        push af

        ;; Guardamos la posicion del enemigo
        ld c, #0x05
        ld a, e_flag(ix)
        sub c
        jr nz, saltaEnemigo68

        call entityman_getEntityArray_HL 

        ld a , e_y(ix) 
        ld c, a 
        ld a, e_h(ix)
        add a,c
        inc hl
        inc hl
        sub (hl)
        jr z , saltaEnemigo68 ;;si es 0 no colisiona
        jp m , saltaEnemigo68 ;; si es menor que 0 no colisiona

        ld a, (hl) ;; guardamos pos y de la nave  
        ld c,a
        ld a , #16;; la mitad del height de la nave ANTES 18
        add c
        ld c,a ;; lo guardamos en c
        ld a, e_y(ix) ;;cargamos en a obstaculo y
        ld b,a ; metemos en b lo que hay que restar
        ld a,c ; metemos en a a lo que se va a restar
        sub b
        jr z , saltaEnemigo68 
        jp m , saltaEnemigo68

        ld c , #01
        call entityman_getNumBalasEnemigo_A
        cp c
        jr nc, saltaEnemigo68

        call entityman_getEntityArray_HL 
        ;; IZQUIERDA---------------------
        ;; obstaculo_x + obstaculo_W - nave_x <= 0
        ld a , e_x(ix)  ;; guardamos la posicion x del obstaculo
        ld c,a ;; lo metemos en c
        ld a, e_w(ix) ;;guardamos la mitad de la anchura del obstaculo ANTES 18
        add a,c
        inc hl ;; incrementamos 1 hl porque apuntaba al primer byte (flag)
        sub (hl)
        jr z , saltaizq ;;si es 0 no colisiona
        jp m , saltaizq;; si es menor que 0 no colisiona

        ld hl, #enemigoBalaIzq
        call entityman_create_balaEnemigoIzq
        jp saltaEnemigo68

        saltaizq:
        ld hl, #enemigoBala
        call entityman_create_balaEnemigo
        
    saltaEnemigo68:
        pop af
        dec a
        ret z
        ld bc, #sizeof_e
        add ix, bc

    jp _loopEnemigoDispara

    ret

;;********************************COLISIONES*****************************************************************

checkColision_nave::
   _colloop:
        dec a
        ret z
        ld bc, #sizeof_e
        add ix, bc
        push af
        call entityman_getEntityArray_HL ;;Guardamos la posicion de la nave que es la primera entidad
        
        ld c, #0x02
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jr z, haSaltado

        ld c, #0x05
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jr z, haSaltado

        ld c, #0x08
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jr z, haSaltado

        ld c, #0x07
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jr z, haSaltado

        ld c, #0x06
        ld a, e_flag(ix)
        ;;cp c
        sub c
        jr nz, salta3
        ;;COMPROBACIONES
        ;; IZQUIERDA---------------------
        ;; obstaculo_x + obstaculo_W - nave_x <= 0
        haSaltado:
        ld a , e_x(ix)  ;; guardamos la posicion x del obstaculo
        ld c,a ;; lo metemos en c
        ld a, e_w(ix) ;;guardamos la mitad de la anchura del obstaculo ANTES 18
        add a,c
        inc hl ;; incrementamos 1 hl porque apuntaba al primer byte (flag)
        ld b, #02
        sub (hl)
        sub b
        jr z , salta3 ;;si es 0 no colisiona
        jp m , salta3;; si es menor que 0 no colisiona

        ;; DERECHA---------------------
        ;; nave_x + nave_w - obstaculo_x <= 0
        ld a, (hl) ;; guardamos pos x de la nave  
        ld c,a
        ld a , #12 ;; la mitad del width de la nave ANTES 18
        add c
        sub b
        ld c,a ;; lo guardamos en c
        ld a, e_x(ix) ;;cargamos en a obstaculo x
        ld b,a ; metemos en b lo que hay que restar
        ld a,c ; metemos en a a lo que se va a restar
        sub b
        jr z , salta3 ;;si es 0 no colisiona
        jp m , salta3;; si es menor que 0 no colisiona
        

        call entityman_getEntityArray_HL ;;Guardamos la posicion de la nave que es la primera entidad
        ;; obstaculo_y + obstaculo_h - nave_y <= 0
        ld a , e_y(ix)  ;; guardamos la posicion y del obstaculo ANTES 20
        ld c,a ;; lo metemos en c
        ld a, e_h(ix)  ;;guardamos la mitad de la altura del obstaculo ANTES e_h(ix)
        add a,c
        inc hl ;; incrementamos 2 hl porque apuntaba al primer byte (flag), pata apuntar al pos y
        inc hl
        sub (hl)
        ld b, #02
        sub b
        jr z , salta3 ;;si es 0 no colisiona
        jp m , salta3;; si es menor que 0 no colisiona

        ;; nave_y + nave_h - obstaculo_y <= 0
        ld a, (hl) ;; guardamos pos y de la nave  
        ld c,a
        ld a , #16;; la mitad del height de la nave ANTES 18
        add c
        sub b
        ld c,a ;; lo guardamos en c
        ld a, e_y(ix) ;;cargamos en a obstaculo y
        ld b,a ; metemos en b lo que hay que restar
        ld a,c ; metemos en a a lo que se va a restar
        sub b
        jr z , salta3 ;;si es 0 no colisiona
        jp m , salta3;; si es menor que 0 no colisiona

        call entityman_getEntityArray_HL
        
        
        ;; Ponemos el game over ya que ha perdido
            ld a, #01
            ld (gameover), a
            

    salta3:
        pop af
     

    jp _colloop
ret

checkColision_item::
   _colloop3:
        push af
        call entityman_getEntityArray_HL ;;Guardamos la posicion de la nave que es la primera entidad
        ld c, #0x04
        ld a, e_flag(ix)
        sub c
        jp nz, salta6

        ;;COMPROBACIONES
        ;; obstaculo_x + obstaculo_W - nave_x <= 0
        ld a , e_x(ix)  ;; guardamos la posicion x del obstaculo
        ld c,a ;; lo metemos en c
        ld a, e_w(ix) ;;guardamos la mitad de la anchura del obstaculo
        add a,c
        inc hl ;; incrementamos 1 hl porque apuntaba al primer byte (flag)
        sub (hl)
        jp z , salta6 ;;si es 0 no colisiona
        jp m , salta6;; si es menor que 0 no colisiona

        ;; nave_x + nave_w - obstaculo_x <= 0
        ld a, (hl) ;; guardamos pos x de la nave  
        ld c,a
        ld a , #12 ;; la mitad del width de la nave
        add c
        ld c,a ;; lo guardamos en c
        ld a, e_x(ix) ;;cargamos en a obstaculo x
        ld b,a ; metemos en b lo que hay que restar
        ld a,c ; metemos en a a lo que se va a restar
        sub b
        jp z , salta6 ;;si es 0 no colisiona
        jp m , salta6;; si es menor que 0 no colisiona
        

        call entityman_getEntityArray_HL ;;Guardamos la posicion de la nave que es la primera entidad
        ;; obstaculo_y + obstaculo_h - nave_y <= 0
        ld a , e_y(ix)  ;; guardamos la posicion y del obstaculo
        ld c,a ;; lo metemos en c
        ld a, e_h(ix) ;;guardamos la mitad de la altura del obstaculo
        add a,c
        inc hl ;; incrementamos 2 hl porque apuntaba al primer byte (flag), pata apuntar al pos y
        inc hl
        sub (hl)
        jr z , salta6 ;;si es 0 no colisiona
        jp m , salta6;; si es menor que 0 no colisiona

        ;; nave_y + nave_h - obstaculo_y <= 0
        ld a, (hl) ;; guardamos pos y de la nave  
        ld c,a
        ld a , #16;; la mitad del height de la nave
        add c
        ld c,a ;; lo guardamos en c
        ld a, e_y(ix) ;;cargamos en a obstaculo y
        ld b,a ; metemos en b lo que hay que restar
        ld a,c ; metemos en a a lo que se va a restar
        sub b
        jr z , salta6 ;;si es 0 no colisiona
        jp m , salta6;; si es menor que 0 no colisiona

        call rendersys_borraBalaInvalida

        call entityman_getLastElement_ptr
        ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
            bucle4:
                dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                dec a
            jr nz, bucle4

        ld a, (hl)
        ld e_flag(ix), a
        ld (hl), #0x00
        inc hl
        ld a, e_x(ix)
        ld a, (hl)
        ld e_x(ix), a
        inc hl
        ld a, e_y(ix)
        ld a, (hl)
        ld e_y(ix), a
        inc hl
        ld a, (hl)
        ld e_color(ix), a
        inc hl
        ld a, (hl)
        ld e_vel(ix), a
        inc hl
        ld a, (hl)
        ld e_vely(ix), a
        inc hl
        ld a, (hl)
        ld e_h(ix), a
        inc hl
        ld a, (hl)
        ld e_w(ix), a
        inc hl
        ld a, (hl)
        ld e_antx(ix), a
        inc hl
        ld a, (hl)
        ld e_anty(ix), a
        inc hl

        ;;;;---------------------SUMAMOS EL PUNTO POR SALVAR, por coger el item-------------------
        push hl
        call entityman_sumapunto

       
        pop hl
          ;; AQUI TERMINA LA ELIMINACION DEL OBSTACULO
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                bucle7:
                        dec ix
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucle7
                call entityman_destroyItem
                call entityman_setLastElement_ptr
        
        salta6:
            pop af
            dec a
            ret z
            ld bc, #sizeof_e
            add ix, bc
            
    jp _colloop3
ret



;; COMPROBACION COLISION BALA HL balas y IX 
checkColision_bala::
    call entityman_getNumEntities_A
    call entityman_getEntityArray_HL ;; Guardamos la posicion de la nave que es la primera entidad
    ;; b guardara el contador de entidades para obstaculos y d para balas
    ld b, a ;; para las balas

   _colbala:
        dec b ;; decrementamos de primeras d ya que no contamos con la nave
        ret z
        ;; call entityman_getNumBalas_A
        ;; ld e, a ;; para los obstaculos
        inc hl ;; incrementamos hl 10 veces para que apunte al siguiente elemento que no sera la nave
        inc hl
        inc hl
        inc hl 
        inc hl
        inc hl 
        inc hl
        inc hl 
        inc hl
        inc hl
        ld d, h
        ld e, l
        ld c, #0x03
        ld a, (hl)
        sub c
        jr nz, _colbala ;; si no es una bala vuelve a incrementar   
        push bc ;; se mete b en la pila para almacenar el contadior y que no se modifique
        call entityman_getNumEntities_A

        _colloop2:
            ld h, d
            ld l, e
            dec a
            push af      
            ld bc, #sizeof_e
            add ix, bc

            ;; Comprobamos si en ix hay un obstaculo
            ld c, #0x02
            ld a, e_flag(ix)
            sub c
            jp z, salta4Obstaculo

            ;; Comprobamos si en ix hay un enemigo
            ld c, #0x05
            ld a, e_flag(ix)
            sub c
            jp nz, salta4

            salta4Obstaculo: 
            ;;COMPROBACIONES
            ;; obstaculo_x + obstaculo_W - bala_x <= 0
            ld a , e_x(ix)  ;; guardamos la posicion x del obstaculo
            ld c,a ;; lo metemos en c
            ld a, e_w(ix);;guardamos la anchura del obstaculo
            add a,c
            inc hl ;; incrementamos 1 hl porque apuntaba al primer byte (flag)
            sub (hl)
            jp z , salta4 ;;si es 0 no colisiona
            jp m , salta4;; si es menor que 0 no colisiona

            ;; bala_x + bala_w - obstaculo_x <= 0
            ld a, (hl) ;; guardamos pos x de la nave  
            ld c,a
            ld a , #4  ;; width de la nave
            add a,c
            ld c,a ;; lo guardamos en c
            ld a, e_x(ix) ;;cargamos en a obstaculo x
            ld b,a ; metemos en b lo que hay que restar
            ld a,c ; metemos en a a lo que se va a restar
            sub b
            jp z , salta4 ;;si es 0 no colisiona
            jp m , salta4;; si es menor que 0 no colisiona
           
            ;; obstaculo_y + obstaculo_h - bala_y <= 0
            ld a , e_y(ix)  ;; guardamos la posicion y del obstaculo
            ld c,a ;; lo metemos en c
            ld a, e_h(ix) ;;guardamos  la altura del obstaculo
            add a,c
            inc hl ;; incrementamos 1 hl porque apuntaba al segundo byte, pata apuntar al pos y
            sub (hl)
            jp z , salta4 ;;si es 0 no colisiona
            jp m , salta4;; si es menor que 0 no colisiona

            ;; bala_y + bala_h - obstaculo_y <= 0
            ld a, (hl) ;; guardamos pos y de la nave  
            ld c,a
            ld a , #8;;  height de la bala
            add a,c
            ld c,a ;; lo guardamos en c
            ld a, e_y(ix) ;;cargamos en a obstaculo y
            ld b,a ; metemos en b lo que hay que restar
            ld a,c ; metemos en a a lo que se va a restar
            sub b
            jp z , salta4 ;;si es 0 no colisiona
            jp m , salta4;; si es menor que 0 no colisiona

            ;;---------------------------BORRAMOS LA BALA Y EL OBSTACULO IX = obstaculo | HL = bala----------------------------------------------------------------
            dec hl 
            dec hl ;; decrementamos 2 para que apunte al primer byte de la entidad 
            push hl ;; guardamos la bala en la pila

            call rendersys_borraBalaInvalida ;;borra el obstaculo de pantalla

            ;; Cogemos el ultimo elemento para pasarlo a donde estaba el obstaculo
             call entityman_getLastElement_ptr
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                    bucle:
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucle

                ld a, (hl)
                ld e_flag(ix), a
                ld (hl), #0x00
                inc hl
                ld a, e_x(ix)
                ld a, (hl)
                ld e_x(ix), a
                inc hl
                ld a, e_y(ix)
                ld a, (hl)
                ld e_y(ix), a
                inc hl
                ld a, (hl)
                ld e_color(ix), a
                inc hl
                ld a, (hl)
                ld e_vel(ix), a
                inc hl
                ld a, (hl)
                ld e_vely(ix), a
                inc hl
                ld a, (hl)
                ld e_h(ix), a
                inc hl
                ld a, (hl)
                ld e_w(ix), a
                inc hl
                ld a, (hl)
                ld e_antx(ix), a
                inc hl
                ld a, (hl)
                ld e_anty(ix), a
                inc hl

                ;; AQUI TERMINA LA ELIMINACION DEL OBSTACULO
                ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
                bucle2:
                        dec ix
                        dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
                        dec a
                    jr nz, bucle2

                call entityman_setLastElement_ptr
             

                
               pop hl ;;recuperamos la posicion de la bala
             ;;  ld d, h ;; cargo en de el ultimo elemento para pasarlo a la posicion de la bala
             ;;  ld e, l ;; cargo en de el ultimo elemento para pasarlo a la posicion de la bala
             ;;  push de
             ;;  ;; Borramos
             ;;      inc hl
             ;;      inc hl
             ;;      inc hl
             ;;      inc hl
             ;;      inc hl
             ;;      inc hl
             ;;      inc hl
             ;;      inc hl
             ;;      ld de, #0xC000
             ;;      ld a, (hl)
             ;;      ld c, a          ;; carga en c lo que hay en ix mas 1. POSICIOM
             ;;      inc hl
             ;;      ld a, (hl)
             ;;      ld b, a 
             ;;      call cpct_getScreenPtr_asm  ;; Calculo el puntero a pantalla, lo tengo en hl
             ;;      ex de, hl                   ;; intercambio lo que hay en hl a de
             ;;      ld a, #00       
             ;;      ld (de), a
             ;;     
             ;;  pop de

             ;;  ld h,d
             ;;  ld l,e
             ;;  push hl

             ;;  call entityman_getLastElement_ptr ;; cargo la ultima entidad otra vez

             ;;  ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
             ;;  bucle5:
             ;;          dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
             ;;          dec a
             ;;      jr nz, bucle5
             ;; 
             ;;  ld a, (hl)  ;;guardo el flag en a para ponerlo en de
             ;;  ld (de), a
             ;;  ld (hl), #00 ;; lo pongo a 0 en hl
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo x
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo y
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo color
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo velx
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo vely
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo height
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo width
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo ant_x
             ;;  ld (de), a
             ;;  inc hl
             ;;  inc de
             ;;  ld a, (hl)  ;;guardo ant_y
             ;;  ld (de), a
             ;;  inc hl
             ;; 
             ;;  ld a, #sizeof_e  ;; antes habia un 16 por eso daba como un bug que se pintaba la nave de nuevo
             ;;  bucle3:
             ;;          dec hl  ;; decrementamos el num de bytes por entidad para que coja la ultima entidad del array
             ;;          dec a
             ;;      jr nz, bucle3
             ;;  call entityman_setLastElement_ptr
             ;;  call entityman_destroyBala
            ;;jp checkColision_bala

            call entityman_destroyObstaculo
            salta4:
                pop af
                jp nz, _colloop2

                pop bc
    jp _colbala

ret